﻿using Newtonsoft.Json.Linq;
using NLog;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Text;


namespace kintone2pptx2
{
    internal class KintoneLib
    {
        private static Logger logger = LogManager.GetCurrentClassLogger();


        // kintoneコンフィグレーションクラス
        public class conf_class
        {
            public string app_name { get; set; }
            public string domain { get; set; }
            public string space_id { get; set; }
            public string app_id { get; set; }
            public string api_token { get; set; }
            public string user_pw64 { get; set; }
            public bool isGuest { get; set; }

        }


        /// <summary>
        /// kintone API呼び出しの共通関数
        /// </summary>
        /// <param name="sub_url"></param>
        /// <param name="mode"></param>
        /// <param name="mainObj"></param>
        /// <returns></returns>
        private JObject recOperation(conf_class conf, string sub_url, string mode, JObject mainObj)
        {
            logger.Debug(mainObj);
            var client = new HttpClient();
            client.DefaultRequestHeaders.Add("ContentType", "application/json");
            if (conf.api_token == null)
            {
                client.DefaultRequestHeaders.Add("X-Cybozu-Authorization", conf.user_pw64);
                logger.Debug("パスワード認証");
            }
            else
            {
                client.DefaultRequestHeaders.Add("X-Cybozu-API-Token", conf.api_token);
                logger.Debug("APIトークン認証");
            }

            string guest_path = "";
            if (conf.isGuest == true)
            {
                guest_path = $"guest/{conf.space_id}/";
            }

            string url = $"https://{conf.domain}.cybozu.com/k/{guest_path}v1/{sub_url}";
            StringContent content = null;
            if (mainObj != null)
            {
                content = new StringContent(mainObj.ToString(), Encoding.UTF8, @"application/json");
            }

            HttpResponseMessage response = null;
            if (mode == "post")
            {
                response = client.PostAsync(url, content).Result;
            }
            else if (mode == "put")
            {
                response = client.PutAsync(url, content).Result;
            }
            else if (mode == "get")
            {
                response = client.GetAsync(url).Result;
            }
            string result = response.Content.ReadAsStringAsync().Result;
            var objResult = JObject.Parse(result);

            logger.Debug(response.StatusCode);
            logger.Debug(objResult);

            return objResult;
        }


        /// <summary>
        /// kintoneのフィールド情報を取得する
        /// </summary>
        /// <returns></returns>
        public JArray GetFields(conf_class conf)
        {
            string sub_url = $"app/form/fields.json?app={conf.app_id}";
            var objResult = recOperation(conf, sub_url, "get", null);

            var objProperties = (JObject)objResult["properties"];
            string[] typeGroup = { "GROUP", "STATUS_ASSIGNEE", "MODIFIER", "STATUS", "CATEGORY", "UPDATED_TIME", "CREATOR", "CREATED_TIME" };

            var arrFieldCode = new JArray();
            foreach (var item in objProperties)
            {
                if (!typeGroup.Any(str => str == (string)item.Value["type"]))
                {
                    arrFieldCode.Add(item.Value["code"].ToString());
                }
            }

            var arrSorted = arrFieldCode.OrderBy(item => item).ToArray();

            return JArray.FromObject(arrSorted);
        }


        /// <summary>
        /// kintoneのレコードを再帰的に全件取得する
        /// </summary>
        public JArray GetAllRecords(conf_class conf, JArray listFields, string orderField)
        {
            JArray arrRecords = new JArray();
            string flds = string.Join(",", listFields);
            int pageSize = 500;
            int offset = 0;
            int totalCount = 0;

            do
            {
                string filterCond = $"order by {orderField} desc limit {pageSize} offset {offset}";
                string sub_url = $"records.json?app={conf.app_id}&totalCount=true&query={filterCond}&fields={flds}";
                var objResult = recOperation(conf, sub_url, "get", null);
                totalCount = (int)objResult["totalCount"];
                var records = (JArray)objResult["records"];
                arrRecords.Merge(records);
                offset += pageSize; // 次のページを取得するためにoffsetを更新
            } while (offset < totalCount);

            return arrRecords;
        }


        /// <summary>
        /// kintoneレコードを更新する
        /// </summary>
        /// <param name="mainObj"></param>
        public void data_update(conf_class conf, JObject mainObj)
        {
            string sub_url = "records.json";
            recOperation(conf, sub_url, "put", mainObj);
        }


        /// <summary>
        /// kintoneレコードを新規登録する
        /// </summary>
        /// <param name="mainObj"></param>
        public void data_insert(conf_class conf, JObject mainObj)
        {
            string sub_url = "records.json";
            recOperation(conf, sub_url, "post", mainObj);
        }


    }
}
