﻿using ClosedXML.Excel;
using DocumentFormat.OpenXml.Office.Word;
using Newtonsoft.Json.Linq;
using NLog;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text.RegularExpressions;
using System.Windows.Forms;


namespace kintone2pptx2
{
    internal class Offline_lib
    {


        private static Logger logger = LogManager.GetCurrentClassLogger();


        List<string> listFields = new List<string>();

        string key_field = "";
        int key_field_column_index = 0;


        KintoneLib.conf_class conf = new KintoneLib.conf_class();
        JArray kintone_records = new JArray();
        TextBox txtMessage;

        public string UserName { get; set; }
        public string AppName { get; set; }

        /// コンストラクタ
        public Offline_lib(KintoneLib.conf_class _conf, TextBox _txtMessage)
        {
            conf = _conf;
            txtMessage = _txtMessage;
            ReadFieldsDef();
        }


        /// <summary>
        /// ファイル名に使える文字列に変換して返す
        /// </summary>
        /// <param name="fileName"></param>
        /// <returns></returns>
        static public string CleanFileName(string fileName)
        {
            string invalidChars = new string(Path.GetInvalidFileNameChars()) + new string(Path.GetInvalidPathChars());
            Regex regex = new Regex(string.Format("[{0}]", Regex.Escape(invalidChars)));
            return regex.Replace(fileName, "_");
        }


        /// <summary>
        /// kintoneのレコードを再帰的に全件取得する
        /// </summary>
        public void GetAllRecords()
        {
            //Kintone kintone = new Kintone(conf.domain, conf.space_id, conf.app_id, conf.api_token, conf.user_pw64);
            //kintone_records = kintone.GetAllRecords(listFields, key_field);

            //KintoneLib kintoneLib = new KintoneLib();
            //kintone_records = kintoneLib.GetAllRecords(conf, listFields, key_field);

            //txtMessage.AppendText($"{kintone_records.Count()} 件のレコードを取得しました \r\n");
        }


        /// <summary>
        /// フィールドコードをexcelにエクスポートする
        /// </summary>
        public void ExportFields()
        {
            //Kintone kintone = new Kintone(conf.domain, conf.space_id, conf.app_id, conf.api_token, conf.user_pw64);
            KintoneLib kintoneLib = new KintoneLib();
            JArray arrFields = kintoneLib.GetFields(conf);
            //JArray arrFields = kintone.GetFields();

            using (var workbook = new XLWorkbook(@"conf\fields_def_template.xlsx"))
            {
                var worksheet = workbook.Worksheet(1);

                int col_cnt = 2;
                foreach (var item in arrFields)
                {
                    worksheet.Cell(2, col_cnt).Value = (string)item;
                    col_cnt++;
                }

                string output_file = $@"conf\fields_def_{conf.space_id}_{conf.app_id}.xlsx";
                workbook.SaveAs(output_file);

                txtMessage.AppendText($"全フィールド情報を {output_file} に保存しました \r\n");
            }

        }


        /// <summary>
        /// Jsonファイルに保存する
        /// </summary>
        public void SaveJson()
        {
            string output_file = $@"json\{conf.space_id}-{conf.app_id}_{DateTime.Now.ToString("yyyyMMdd_HHmmss")}.json";
            System.IO.File.WriteAllText(output_file, kintone_records.ToString());
        }


        /// <summary>
        /// Excelにデータを保存する
        /// </summary>
        public void SaveExcel()
        {
            using (var workbook = new XLWorkbook(@"conf\data_template.xlsx"))
            {
                var worksheet = workbook.Worksheet(1);

                for (int j = 0; j < listFields.Count; j++)
                {
                    worksheet.Cell(1, j + 1).Value = (string)listFields[j];
                }

                for (int i = 0; i < kintone_records.Count(); i++)
                {
                    for (int j = 0; j < listFields.Count; j++)
                    {
                        string cell_type = (string)kintone_records[i][listFields[j]]["type"];
                        if (cell_type == "DATE")
                        {
                            string date_str = (string)kintone_records[i][listFields[j]]["value"];
                            if (date_str != null)
                            {
                                worksheet.Cell(i + 2, j + 1).Value = Convert.ToDateTime(date_str);
                            }
                        }
                        else
                        {
                            worksheet.Cell(i + 2, j + 1).Value = (string)kintone_records[i][listFields[j]]["value"];
                        }
                    }
                }
                string excel_file_body = CleanFileName($"{conf.space_id}-{conf.app_id}_{conf.app_name}");
                CheckCreateDirectory($@"excel\{excel_file_body}");

                string output_file = $@"excel\{excel_file_body}\{excel_file_body}_{DateTime.Now.ToString("yyyyMMdd_HHmmss")}.xlsx";
                workbook.SaveAs(output_file);

                txtMessage.AppendText($"{output_file} に保存しました \r\n");
            }
        }


        /// <summary>
        /// データExcelを読み込んで差分をkintoneを更新する
        /// </summary>
        /// <param name="excel_file"></param>
        public void ReadDataExcel(string excel_file)
        {
            using (var workbook = new XLWorkbook(excel_file))
            {
                var worksheet = workbook.Worksheet(1);
                var rowCount = worksheet.LastRowUsed().RowNumber();
                var colCount = worksheet.LastColumnUsed().ColumnNumber();
                string message = $"space={conf.space_id}, apl={conf.app_id} {conf.app_name} のデータを更新します。よろしいですか？\r\n";
                JArray arrRecords = new JArray();
                JArray arrLogRecords = new JArray();
                for (int row_cnt = 1; row_cnt < rowCount; row_cnt++)
                {
                    string key = (string)worksheet.Cell(row_cnt + 1, key_field_column_index).Value;
                    JObject objRecord = new JObject();

                    for (int col_cnt = 0; col_cnt < colCount; col_cnt++)
                    {
                        string colname = (string)worksheet.Cell(1, col_cnt + 1).Value;
                        if (1 == 1)
                        {
                            string data2 = GetDataByKey(kintone_records, key, colname);
                            var type2 = GetTypeByKey(kintone_records, key, colname);
                            string data = "";
                            if (worksheet.Cell(row_cnt + 1, col_cnt + 1).IsEmpty() == false)
                            {
                                if (type2 == "DATE")
                                {
                                    data = Convert.ToDateTime(worksheet.Cell(row_cnt + 1, col_cnt + 1).GetString()).ToString("yyyy-MM-dd");
                                }
                                else
                                {
                                    data = worksheet.Cell(row_cnt + 1, col_cnt + 1).GetString();
                                }
                            }

                            if (data != data2)
                            {
                                objRecord[colname] = new JObject(
                                    new JProperty("value", data)
                                );
                                message += $"{key} : {colname} {data2} → {data}\r\n";
                                logger.Debug($"{UserName} {AppName} {key} : {colname} {data2} → {data}");

                                JObject objTerm2 = new JObject();
                                objTerm2["入力者"] = new JObject { ["value"] = UserName };
                                objTerm2["アプリ"] = new JObject { ["value"] = AppName };
                                objTerm2["キー"] = new JObject { ["value"] = key };
                                objTerm2["フィールド"] = new JObject { ["value"] = colname };
                                objTerm2["変更前"] = new JObject { ["value"] = data2 };
                                objTerm2["変更後"] = new JObject { ["value"] = data };
                                //logger.Debug(objTerm2);
                                arrLogRecords.Add(objTerm2);

                            }
                        }
                    }
                    logger.Debug(arrLogRecords);

                    if (objRecord.Count > 0)
                    {
                        JObject updateKey1 = new JObject(
                            new JProperty("field", key_field),
                            new JProperty("value", key)
                        );

                        JObject obj2 = new JObject(
                            new JProperty("updateKey", updateKey1),
                            new JProperty("record", objRecord)
                        );
                        arrRecords.Add(obj2);
                    }
                }

                if (arrRecords.Count > 0)
                {
                    JObject mainObj = new JObject(
                         new JProperty("app", conf.app_id),
                        new JProperty("records", arrRecords)
                    );

                    Form2 form2 = new Form2(message);
                    if (form2.ShowDialog() == DialogResult.OK)
                    {
                        //Kintone kintone = new Kintone(conf.domain, conf.space_id, conf.app_id, conf.api_token, conf.user_pw64);
                        KintoneLib kintoneLib = new KintoneLib();
                        kintoneLib.data_update(conf, mainObj);

                        //kintone.data_update(mainObj);
                        txtMessage.AppendText(message + Environment.NewLine);
                        txtMessage.AppendText("更新しました" + Environment.NewLine);
                        logger.Debug(mainObj.ToString());



                        if (arrLogRecords.Count > 0)
                        {
                            JObject log_app = JObject.Parse(File.ReadAllText(@"log_app.json"));

                            JObject mainObj2 = new JObject(
                                new JProperty("app", (int)log_app["app_id"]),
                               new JProperty("records", arrLogRecords));

                            Kintone kintone2 = new Kintone(conf.domain,
                                (int)log_app["space_id"],
                                (int)log_app["app_id"],
                                (string)log_app["api_token"], null);
                            kintone2.data_insert(mainObj2);
                        }

                    }
                    else
                    {
                        txtMessage.AppendText("cancelしました" + Environment.NewLine);
                    }
                }
                else
                {
                    txtMessage.AppendText("変更がありませんでした" + Environment.NewLine);
                }

            }
        }


        /// <summary>
        /// フィールド定義excelを読み込む
        /// </summary>
        private void ReadFieldsDef()
        {
            using (var workbook = new XLWorkbook(@"conf\fields_def.xlsx"))
            {
                var worksheet = workbook.Worksheet(1);
                var colCount = worksheet.LastColumnUsed().ColumnNumber();

                listFields.Clear();
                key_field = "";

                for (int col_cnt = 0; col_cnt < colCount; col_cnt++)
                {
                    string colname = (string)worksheet.Cell(2, col_cnt + 1).Value;
                    if (col_cnt > 0)
                    {
                        listFields.Add(colname);
                        if (worksheet.Cell(1, col_cnt + 1).IsEmpty() == false && (string)worksheet.Cell(1, col_cnt + 1).Value == "*")
                        {
                            key_field = colname;
                            key_field_column_index = col_cnt;
                        }
                    }
                }
            }
        }


        /// <summary>
        /// Jsonデータからキーとフィールドを指定してデータを取得する
        /// </summary>
        /// <param name="data"></param>
        /// <param name="key"></param>
        /// <param name="colname"></param>
        /// <returns></returns>
        private string GetDataByKey(JArray data, string key, string colname)
        {
            JObject targetObject = data.Children<JObject>().FirstOrDefault(o => o[key_field]["value"] != null && o[key_field]["value"].ToString() == key);
            if (targetObject != null)
            {
                return targetObject[colname]["value"].ToString();
            }
            else
            {
                return null;
            }
        }


        /// <summary>
        /// Jsonデータからキーとフィールドを指定してtypeを取得する
        /// </summary>
        /// <param name="data"></param>
        /// <param name="key"></param>
        /// <param name="colname"></param>
        /// <returns></returns>
        private string GetTypeByKey(JArray data, string key, string colname)
        {
            JObject targetObject = data.Children<JObject>().FirstOrDefault(o => o[key_field]["value"] != null && o[key_field]["value"].ToString() == key);
            if (targetObject != null)
            {
                return targetObject[colname]["type"].ToString();
            }
            else
            {
                return null;
            }
        }


        /// <summary>
        /// ディレクトリがなければ作成する
        /// </summary>
        /// <param name="folderPath"></param>
        public void CheckCreateDirectory(string folderPath)
        {
            if (!Directory.Exists(folderPath))
            {
                try
                {
                    Directory.CreateDirectory(folderPath);
                    Debug.WriteLine("フォルダーが作成されました。");
                }
                catch (Exception ex)
                {
                    Debug.WriteLine($"エラーが発生しました: {ex.Message}");
                }
            }
            else
            {
                Debug.WriteLine("フォルダーはすでに存在します。");
            }
        }



    }
}
