﻿using ClosedXML.Excel;
using Microsoft.Office.Core;
using Microsoft.Office.Interop.PowerPoint;
using Newtonsoft.Json.Linq;
using NLog;
using System;
using System.Data.SQLite;
using System.Diagnostics;
using System.Linq;
using System.Net.Http;
using System.Runtime.InteropServices;
using System.Text;


namespace kintone2pptx2
{
    internal class kintone2pptx_lib
    {
        private static Logger logger = LogManager.GetCurrentClassLogger();

        string domain = null;
        string space_id = null;
        string app_id = null;
        string api_token = null;
        bool isGuest = false;
        string user_pw64 = null;
        JArray arrSelectedFields = new JArray();
        string query = null;

        JArray arrFields = null;
        JObject objFieldsDetail = null;
        JArray arrRecords = null;



        /// <summary>
        /// excel設定ファイルを読み込み、jsonファイルに書き出す
        /// </summary>
        public void ReadConf(string excel_file)
        {
            using (var workbook = new XLWorkbook(excel_file))
            {
                // kintone関係
                var worksheet = workbook.Worksheet("kintone");
                JObject objTemp = new JObject();
                objTemp["domain"] = worksheet.Cell(2, 2).Value.ToString();
                objTemp["space_id"] = worksheet.Cell(3, 2).Value.ToString();
                objTemp["app_id"] = worksheet.Cell(4, 2).Value.ToString();
                objTemp["api_token"] = worksheet.Cell(5, 2).Value.ToString();
                objTemp["query"] = worksheet.Cell(7, 2).Value.ToString();
                objTemp["isGuest"] = worksheet.Cell(8, 2).Value.ToString();

                // kintoneでselectするフィールドの読み込み
                arrSelectedFields.Clear();
                int col_cnt = 2;
                while (true)
                {
                    string val = worksheet.Cell(6, col_cnt).Value.ToString();
                    if (val == "" || worksheet.Cell(6, col_cnt) == null || col_cnt > 500)
                    {
                        break;
                    }
                    arrSelectedFields.Add(val);
                    col_cnt++;
                }
                objTemp["fields"] = arrSelectedFields;

                System.IO.File.WriteAllText(@"conf\kintone設定.json", objTemp.ToString());

                worksheet = workbook.Worksheet("パワーポイント");
                objTemp = new JObject();
                objTemp["template"] = worksheet.Cell(2, 2).Value.ToString();
                System.IO.File.WriteAllText(@"conf\powerpoint.json", objTemp.ToString());

                // SQLの検索式の読み込み
                worksheet = workbook.Worksheet("検索式");
                objTemp = new JObject();

                int rowCount = worksheet.LastRowUsed().RowNumber();
                for (int row_cnt = 2; row_cnt <= rowCount; row_cnt++)
                {
                    objTemp[worksheet.Cell(row_cnt, 1).Value.ToString()] = worksheet.Cell(row_cnt, 2).Value.ToString();
                }

                System.IO.File.WriteAllText(@"conf\検索式.json", objTemp.ToString());
            }
        }


        /// <summary>
        /// 設定ファイルのjsonを読み込み、内部変数に格納する
        /// </summary>
        public void LoadConf()
        {
            JObject conf = JObject.Parse(System.IO.File.ReadAllText(@"conf\kintone設定.json"));
            domain = conf["domain"].ToString();
            space_id = conf["space_id"].ToString();
            app_id = conf["app_id"].ToString();
            api_token = conf["api_token"].ToString();
            query = conf["query"].ToString();
            arrSelectedFields = (JArray)conf["fields"];
            if (conf["isGuest"].ToString() == "")
            {
                isGuest = false;
            }
            else
            {
                isGuest = true;
            }
        }


        /// <summary>
        /// kintoneのフィールド情報を取得し、jsonファイルに書き出す
        /// </summary>
        public void GetFields()
        {
            string sub_url = $"app/form/fields.json?app={app_id}";
            var objResult = kintoneApi(sub_url, "get", null);
            objFieldsDetail = (JObject)objResult["properties"];//fieldsの詳細情報

            string[] arrSpecialType = {
                "GROUP", "STATUS_ASSIGNEE", "MODIFIER", "STATUS", "CATEGORY", "UPDATED_TIME", "CREATOR", "CREATED_TIME"
            };  //除外するフィールドのタイプ

            // 全フィールドのコードを取得しソートする
            var arrFieldCode = new JArray();
            foreach (var item in objFieldsDetail)
            {
                if (!arrSpecialType.Any(str => str == (string)item.Value["type"]))
                {
                    arrFieldCode.Add(item.Value["code"].ToString());
                }
            }

            var arrSorted = arrFieldCode.OrderBy(item => item).ToArray();

            System.IO.File.WriteAllText(@"conf\全フィールド詳細.json", objFieldsDetail.ToString());
            System.IO.File.WriteAllText(@"conf\全フィールド.json", JArray.FromObject(arrSorted).ToString());

            //JArray.FromObject(arrSorted)の値をexcelに書き込む
            using (var workbook = new XLWorkbook())
            {
                var worksheet = workbook.Worksheets.Add("全フィールド");
                for (int i = 0; i < arrSorted.Length; i++)
                {
                    worksheet.Cell(1, i + 1).Value = arrSorted[i].ToString();
                }
                workbook.SaveAs(@"conf\全フィールド.xlsx");
            }
        }


        /// <summary>
        /// kintone rest apiを実行する
        /// </summary>
        /// <param name="sub_url"></param>
        /// <param name="mode"></param>
        /// <param name="mainObj"></param>
        /// <returns></returns>
        private JObject kintoneApi(string sub_url, string mode, JObject mainObj)
        {
            logger.Debug(mainObj);
            var client = new HttpClient();
            client.DefaultRequestHeaders.Add("ContentType", "application/json");
            if (api_token == null || api_token=="")
            {
                client.DefaultRequestHeaders.Add("X-Cybozu-Authorization", user_pw64);
                logger.Debug("パスワード認証");
            }
            else
            {
                client.DefaultRequestHeaders.Add("X-Cybozu-API-Token", api_token);
                logger.Debug("APIトークン認証");
            }

            string guest_path = "";
            if (isGuest == true)
            {
                guest_path = $"guest/{space_id}/";
            }

            string url = $"https://{domain}.cybozu.com/k/{guest_path}v1/{sub_url}";
            StringContent content = null;
            if (mainObj != null)
            {
                content = new StringContent(mainObj.ToString(), Encoding.UTF8, @"application/json");
            }

            HttpResponseMessage response = null;
            if (mode == "post")
            {
                response = client.PostAsync(url, content).Result;
            }
            else if (mode == "put")
            {
                response = client.PutAsync(url, content).Result;
            }
            else if (mode == "get")
            {
                response = client.GetAsync(url).Result;
            }
            string result = response.Content.ReadAsStringAsync().Result;
            var objResult = JObject.Parse(result);

            //logger.Debug(response.StatusCode);
            //logger.Debug(objResult);

            return objResult;
        }


        /// <summary>
        /// フィールド情報のjsonファイルを読み込む
        /// </summary>
        public void LoadFields()
        {
            objFieldsDetail = JObject.Parse(System.IO.File.ReadAllText(@"conf\全フィールド詳細.json"));
            arrFields = JArray.Parse(System.IO.File.ReadAllText(@"conf\全フィールド.json"));
        }


        /// <summary>
        /// 再帰的に全レコードを取得する
        /// </summary>
        /// <param name="orderField"></param>
        /// <returns></returns>
        public JArray GetAllRecords(string orderField)
        {
            JArray arrRecords = new JArray();
            string flds = "";
            if (arrSelectedFields.Count() == 0)
            {
                flds = string.Join(",", arrFields);
            }
            else
            {
                flds = string.Join(",", arrSelectedFields);
            }

            int pageSize = 500;
            int offset = 0;
            int totalCount = 0;

            do
            {
                string filterCond = $"{query} order by {orderField} desc limit {pageSize} offset {offset}";

                string sub_url = $"records.json?app={app_id}&totalCount=true&query={filterCond}&fields={flds}";
                var objResult = kintoneApi(sub_url, "get", null);
                totalCount = (int)objResult["totalCount"];
                var records = (JArray)objResult["records"];
                arrRecords.Merge(records);
                offset += pageSize; // 次のページを取得するためにoffsetを更新
            } while (offset < totalCount);

            return arrRecords;
        }


        /// <summary>
        /// 全レコードを取得し、jsonファイルに書き出す
        /// </summary>
        public void GetRecords()
        {
            arrRecords = GetAllRecords("レコード番号");
            System.IO.File.WriteAllText(@"conf\records.json", arrRecords.ToString());
        }


        /// <summary>
        /// jsonファイルから全レコードを読み込む
        /// </summary>
        public void loadRecords()
        {
            arrRecords = JArray.Parse(System.IO.File.ReadAllText(@"conf\records.json"));
        }


        /// <summary>
        /// SQLiteのテーブルを作成する
        /// </summary>
        public void CreateTable()
        {
            string table_name = $"tbl{app_id}";

            SQLiteConnection con = new SQLiteConnection("Data Source=database.sqlite;Version=3;");
            con.Open();

            SQLiteCommand com = new SQLiteCommand($"drop table if exists {table_name}", con);
            com.ExecuteNonQuery();

            try
            {
                string sql = $@"create table if not exists {table_name} (" + Environment.NewLine;

                int cnt = 0;
                foreach (var field in arrFields)
                {
                    string field_type = "text";
                    string tp = (string)objFieldsDetail[field.ToString()]["type"];
                    if (tp == "NUMBER")
                    {
                        field_type = "NUMBER";
                    }
                    else if (tp == "DATETIME" || tp == "DATE")
                    {
                        field_type = "TIMESTAMP";
                    }
                    else
                    {
                        field_type = "TEXT";
                    }

                    if (cnt == 0)
                    {
                        sql += $@"{field} {field_type}" + Environment.NewLine;
                    }
                    else
                    {
                        sql += $@", {field} {field_type}" + Environment.NewLine;
                    }

                    cnt++;
                }
                sql += ")";
                com = new SQLiteCommand(sql, con);
                var sdr = com.ExecuteNonQuery();
            }
            finally
            {
                con.Close();
            }

        }


        /// <summary>
        /// UTCの日時をJSTに変換する
        /// </summary>
        /// <param name="utcTime"></param>
        /// <returns></returns>
        /// <exception cref="ArgumentException"></exception>
        public static string ConvertUtcToJst(string utcTime)
        {
            DateTime utcDateTime;
            if (!DateTime.TryParse(utcTime, out utcDateTime))
            {
                throw new ArgumentException("Invalid UTC time format");
            }

            TimeZoneInfo jstZone = TimeZoneInfo.FindSystemTimeZoneById("Tokyo Standard Time");
            DateTime jstDateTime = TimeZoneInfo.ConvertTimeFromUtc(utcDateTime, jstZone);

            return jstDateTime.ToString("yyyy-MM-dd HH:mm:ss");
        }


        /// <summary>
        /// DBにデータを挿入する
        /// </summary>
        public void insertData()
        {
            string table_name = $"tbl{app_id}";

            SQLiteConnection con = new SQLiteConnection("Data Source=database.sqlite;Version=3;");
            con.Open();
            SQLiteCommand com = new SQLiteCommand($"delete from {table_name}", con);
            com.ExecuteNonQuery();

            var data2 = ConvertJArrayToSimplifiedJArray(arrRecords);

            foreach (JObject item in data2)
            {
                string fields_str = "";
                string values = "( ";
                foreach (string key in arrFields)
                {
                    string val = (string)item[key];
                    fields_str += key + ",";
                    values += $"\"{val}\",";
                }
                fields_str = fields_str.Substring(0, fields_str.Length - 1);
                values = values.Substring(0, values.Length - 1) + " )";
                string sql = $@"insert into {table_name} ({fields_str}) values {values}";

                com = new SQLiteCommand(sql, con);
                var sdr = com.ExecuteNonQuery();

            }
            con.Close();
        }


        private JArray ConvertJArrayToSimplifiedJArray(JArray jsonArray)
        {
            JArray simplifiedJsonArray = new JArray();

            foreach (var item in jsonArray)
            {
                JObject simplifiedItem = new JObject();

                foreach (JProperty property in item.Children<JProperty>())
                {
                    if ((string)property.Value["type"] == "DATETIME" && property.Value["value"].ToString() != "")
                    {
                        simplifiedItem[property.Name] = ConvertUtcToJst(property.Value["value"].ToString());
                    }
                    else
                    {
                        simplifiedItem[property.Name] = property.Value["value"];
                    }
                }

                simplifiedJsonArray.Add(simplifiedItem);
            }

            return simplifiedJsonArray;
        }


        /// <summary>
        /// SQLを実行し、結果をjsonファイルに書き出す
        /// </summary>
        public void sqlExecute()
        {
            JObject objSql = JObject.Parse(System.IO.File.ReadAllText(@"conf\検索式.json"));

            JObject objOut = new JObject();
            foreach (var item in objSql)
            {
                string sql = item.Value.ToString().Replace("table", $"tbl{app_id}");
                var count = selecOne(sql);
                objOut[item.Key] = $"{count}";
            }

            System.IO.File.WriteAllText(@"conf\ppt用データ.json", objOut.ToString());
        }


        /// <summary>
        /// SQLite3のselectを実行し、スカラー値を返す
        /// </summary>
        /// <param name="sql"></param>
        /// <returns></returns>
        public int selecOne(string sql)
        {
            int count = -1;

            SQLiteConnection con = new SQLiteConnection("Data Source=database.sqlite;Version=3;");
            con.Open();
            using (SQLiteCommand command = new SQLiteCommand(sql, con))
            {
                // クエリを実行し、結果のスカラー値を取得
                count = Convert.ToInt32(command.ExecuteScalar());
            }

            return count;
        }


        /// <summary>
        /// パワーポイントを作成する
        /// </summary>
        public void createPpt()
        {
            JObject confPptx = JObject.Parse(System.IO.File.ReadAllText(@"conf\powerpoint.json"));
            JObject ppt_data = JObject.Parse(System.IO.File.ReadAllText(@"conf\ppt用データ.json"));

            string exe_path = System.IO.Path.GetDirectoryName(System.Windows.Forms.Application.ExecutablePath);

            string template = (string)confPptx["template"];
            // templateを拡張子と分割する
            string[] template_ext = template.Split('.');

            string file_body = template_ext[0];
            string file_ext = template_ext[1];

            //現在の日時を文字列変数に格納
            string dt = DateTime.Now.ToString("yyyyMMdd_HHmmss");
            string out_file = $"{file_body}_{dt}.{file_ext}";

            string file1 = exe_path + $@"\template\{template}";
            string file2 = exe_path + $@"\output\{out_file}";

            //powerpoint_lib ppt = new powerpoint_lib();
            ReplaceText(ppt_data, file1, file2);
        }


        /// <summary>
        /// パワーポイントのテキストを置換する
        /// </summary>
        /// <param name="ppt_data"></param>
        /// <param name="file1"></param>
        /// <param name="file2"></param>
        public void ReplaceText(JObject ppt_data, string file1, string file2)
        {

            var pptApp = new Microsoft.Office.Interop.PowerPoint.Application();

            try
            {
                Microsoft.Office.Interop.PowerPoint.Presentation presentation = pptApp.Presentations.Open(file1, MsoTriState.msoFalse, MsoTriState.msoFalse, MsoTriState.msoFalse);

                foreach (Microsoft.Office.Interop.PowerPoint.Shape sp in presentation.Slides[1].Shapes)
                {
                    if (sp.HasTable == MsoTriState.msoTrue)
                    {
                        Table table = sp.Table;
                        // テーブルが見つかった場合の処理

                        for (int i = 1; i <= table.Rows.Count; i++)
                        {
                            for (int j = 1; j <= table.Columns.Count; j++)
                            {
                                var tgt = table.Cell(i, j).Shape.TextFrame.TextRange.Text;
                                foreach (var kv in ppt_data.Properties())
                                {
                                    if (tgt == kv.Name)
                                    {
                                        table.Cell(i, j).Shape.TextFrame.TextRange.Text = (string)kv.Value;
                                    }
                                }
                                Debug.WriteLine(tgt);
                            }
                        }

                        break; // すぐにループを終了する場合
                    }
                    Debug.WriteLine(sp.Name);
                }

                presentation.SaveAs(file2);
                presentation.Close();
                pptApp.Quit();
            }
            catch (Exception ex)
            {
                Debug.WriteLine($"エラー: {ex.Message}");
            }
            finally
            {
                // PowerPointを終了
                ReleaseObject(pptApp);
            }

            // COMオブジェクトの解放を確認する
            GC.Collect();
            GC.WaitForPendingFinalizers();

        }


        /// <summary>
        /// comオブジェクトの解放
        /// </summary>
        /// <param name="obj"></param>
        private static void ReleaseObject(object obj)
        {
            try
            {
                Marshal.ReleaseComObject(obj);
                obj = null;
            }
            catch (Exception ex)
            {
                Debug.WriteLine($"COMオブジェクトの解放中にエラーが発生しました: {ex.Message}");
            }
        }


    }
}
